"use strict";
require("dotenv").config({});
const redis = require("redis");
const {promisify} = require("util");
const redisClient = redis.createClient(
    {
        host: process.env.REDIS_HOST || "127.0.0.1",
        port: process.env.REDIS_PORT || "6379"
    }
);

const password = process.env.REDIS_PASSWORD;
if(password && password != "null"){
    redisClient.auth(password, (err,res) => {
        console.log("res",res);
        console.log("err",err);
    });
}

redisClient.getAsync = promisify(redisClient.get).bind(redisClient);
redisClient.setAsync = promisify(redisClient.set).bind(redisClient);
redisClient.lpushAsync = promisify(redisClient.lpush).bind(redisClient);
redisClient.lrangeAsync = promisify(redisClient.lrange).bind(redisClient);
redisClient.llenAsync = promisify(redisClient.llen).bind(redisClient);
redisClient.lremAsync = promisify(redisClient.lrem).bind(redisClient);
redisClient.lsetAsync = promisify(redisClient.lset).bind(redisClient);
redisClient.hmsetAsync = promisify(redisClient.hmset).bind(redisClient);
redisClient.hmgetAsync = promisify(redisClient.hmget).bind(redisClient);
redisClient.hgetAsync = promisify(redisClient.hget).bind(redisClient);
redisClient.hsetAsync = promisify(redisClient.hset).bind(redisClient);
redisClient.hgetAllAsync = promisify(redisClient.hgetall).bind(redisClient);
redisClient.hdelAsync = promisify(redisClient.hdel).bind(redisClient);
redisClient.clear = promisify(redisClient.del).bind(redisClient);

const remember = async (key, value, seconds = null) => {
    try{
        const res = JSON.parse(await redisClient.getAsync(key));
        if(res)
            return res;
        value =  await value();
        if(seconds == null){
            redisClient.set(key, JSON.stringify(value))
        }
        else
            redisClient.setex(key, seconds, JSON.stringify(value));
        return value;
        // return data;
    }catch (e) {
        debug(e);
        return null;
    }
};
redisClient.remember = promisify(remember).bind(redisClient);

redisClient.on("connected", function () {
    console.log("Redis is connected");
});
redisClient.on("error", function (err) {
    console.log("Redis error.", err);
    console.log(err.message, err, {}, true);
    // process.exit(1);
});

setInterval(function() {
    console.log(`Keeping alive ${process.env.APP_NAME}`);
    redisClient.set('ping', 'pong');
}, 1000 * 60 * 4);


global.cache = redisClient;
module.exports = redisClient;
